// ListOf.h
// by Eric Berdahl
// Copyright 1996,1997 Eric Berdahl
// All rights reserved
//
// For licensing instructions, see Betelgeuse.h


#pragma once


// Includes


#include <SupportKit.h>


// Classes defined here


template <class AType> class ListOf;


//------------------------------------------------------------------------------
//	ListOf interface
//------------------------------------------------------------------------------


template <class AType>
class ListOf
{
public:
					ListOf(long itemsPerBlock = 20);
					ListOf(const ListOf<AType>& copy);

	virtual			~ListOf();

	ListOf<AType>&	operator=(const ListOf<AType>& assign);

	bool			AddItem(AType* item);
	bool			AddItem(AType* item, long atIndex);

	bool			AddList(ListOf<AType>& newItems);
	bool			AddList(ListOf<AType>& newItems, long atIndex);

	bool			RemoveItem(AType* item);
	AType*			RemoveItem(long index);

	AType*			ItemAt(long) const;
	AType*			ItemAtFast(long) const;

	long			IndexOf(AType* item) const;

	AType*			FirstItem() const;
	AType*			LastItem() const;

	bool			HasItem(AType* item) const;
	long			CountItems() const;
	void			MakeEmpty();
	bool			IsEmpty() const;

	typedef bool (*IteratorFn)(AType* obj);
	typedef bool (*IteratorParamFn)(AType* obj, void* param);

	void			DoForEach(IteratorFn func);
	void			DoForEach(IteratorParamFn func, void* param);

	AType*			Items() const;

	typedef int (*ComparisonFn)(const AType* a, const AType* b);

	void			SortItems(ComparisonFn cmp);

private:
	BList			fList;

	typedef bool (*BListIteratorFn)(void* obj);
	typedef bool (*BListIteratorParamFn)(void* obj, void* param);
	typedef int (*BListComparisonFn)(const void* a, const void* b);
};


//------------------------------------------------------------------------------
//	ListOf implementation
//------------------------------------------------------------------------------


template <class AType>
inline
ListOf<AType>::ListOf(long itemsPerBlock) :
	fList(itemsPerBlock)
{
}


template <class AType>
inline
ListOf<AType>::ListOf(const ListOf<AType>& copy) :
	fList(copy.fList)
{
}


template <class AType>
/* virtual */
ListOf<AType>::~ListOf()
{
}


template <class AType>
inline ListOf<AType>&
ListOf<AType>::operator=(const ListOf<AType>& assign)
{
	fList = assign.fList;
	return *this;
}


template <class AType>
inline bool
ListOf<AType>::AddItem(AType* item)
{
	return fList.AddItem(reinterpret_cast<void*>(item));
}


template <class AType>
inline bool
ListOf<AType>::AddItem(AType* item, long atIndex)
{
	return fList.AddItem(reinterpret_cast<void*>(item), atIndex);
}


template <class AType>
inline bool
ListOf<AType>::AddList(ListOf<AType>& newItems)
{
	return fList.AddList(&newItems.fList);
}


template <class AType>
inline bool
ListOf<AType>::AddList(ListOf<AType>& newItems, long atIndex)
{
	return fList.AddList(&newItems.fList, atIndex);
}


template <class AType>
inline bool
ListOf<AType>::RemoveItem(AType* item)
{
	return fList.RemoveItem(reinterpret_cast<void*>(item));
}


template <class AType>
inline AType*
ListOf<AType>::RemoveItem(long index)
{
	return reinterpret_cast<AType*>(fList.RemoveItem(index));
}


template <class AType>
inline AType*
ListOf<AType>::ItemAt(long index) const
{
	return reinterpret_cast<AType*>(fList.ItemAt(index));
}


template <class AType>
inline AType*
ListOf<AType>::ItemAtFast(long index) const
{
	return reinterpret_cast<AType*>(fList.ItemAtFast(index));
}


template <class AType>
inline long
ListOf<AType>::IndexOf(AType* item) const
{
	return fList.IndexOf(reinterpret_cast<void*>(item));
}


template <class AType>
inline AType*
ListOf<AType>::FirstItem() const
{
	return reinterpret_cast<AType*>(fList.FirstItem());
}


template <class AType>
inline AType*
ListOf<AType>::LastItem() const
{
	return reinterpret_cast<AType*>(fList.LastItem());
}


template <class AType>
inline bool
ListOf<AType>::HasItem(AType* item) const
{
	return fList.HasItem(reinterpret_cast<void*>(item));
}


template <class AType>
inline long
ListOf<AType>::CountItems() const
{
	return fList.CountItems();
}


template <class AType>
inline void
ListOf<AType>::MakeEmpty()
{
	fList.MakeEmpty();
}


template <class AType>
inline bool
ListOf<AType>::IsEmpty() const
{
	return fList.IsEmpty();
}


template <class AType>
inline void
ListOf<AType>::DoForEach(IteratorFn iter)
{
	fList.DoForEach((BListIteratorFn) func);
}


template <class AType>
inline void
ListOf<AType>::DoForEach(IteratorParamFn func, void* param)
{
	fList.DoForEach((BListIteratorParamFn) func, param);
}


template <class AType>
inline AType*
ListOf<AType>::Items() const
{
	return reinterpret_cast<AType*>(fList.Items());
}


template <class AType>
inline void
ListOf<AType>::SortItems(ComparisonFn cmp)
{
	fList.SortItems((BListComparisonFn) cmp);
}
