// AtomicLong.PPC.cp
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved
//
// For licensing instructions, see Betelgeuse.h
// For usage instructions, see Synchronization.h


#include "Synchronization.h"
#include "SynchronizationInternal.h"


//------------------------------------------------------------------------------
//	AtomicLong implementation
//------------------------------------------------------------------------------


#pragma export on


long
AtomicLong::Get() const
{
		// On PPC, long reads are atomic
	return fValue;
}


void
AtomicLong::Set(long newValue)
{
		// On PPC, long writes are atomic
	fValue = newValue;
}


asm
bool
AtomicLong::SetConditional(register long oldValue, register long newValue)
{
#if 1
	// register usage on entry
	// r3 - this
	// r4 - oldValue
	// r5 - newValue
	//
	// register usage
	// r3 - result
	// r4 - oldValue
	// r5 - newValue
	// r6 - address of fValue
	// r7 - current value of fValue

	addi	r6,r3,AtomicLong.fValue
	li		r3,0

@loop
	lwarx	r7,r0,r6
	cmpw	r7,r4
	bne-	@exit
	stwcx.	r5,r0,r6
	bne-	@loop
	li		r3,1
@exit
	blr
#else
	bool result;

	ReserveEntryPPC theReserve(&fValue);
	bool keepTrying = true;
	while (keepTrying)
	{
		result = (theReserve.Get() == oldValue);
		if (result)
		{
			keepTrying = !theReserve.Set(newValue);
		}
	}

	return result;
#endif
}


#pragma export off
