// CheapReadWriteLock.Be.cp
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved
//
// For licensing instructions, see Betelgeuse.h
// For usage instructions, see Synchronization.h


#include "Exceptions.h"
#include "Synchronization.h"


//------------------------------------------------------------------------------
//	CheapReadWriteLock's Be-specific implementation
//------------------------------------------------------------------------------


void
CheapReadWriteLock::PlatformConstructor()
{
	char theName[B_OS_NAME_LENGTH];
	::sprintf(theName, "CRWLock-reader-%X", (long) this);

	fReaderCondition = ::create_sem(0, theName);
	if (fReaderCondition < B_NO_ERROR)
	{
		::FailBeError(fReaderCondition);
	}

	::sprintf(theName, "CRWLock-writer-%X", (long) this);

	fWriterCondition = ::create_sem(0, theName);
	if (fWriterCondition < B_NO_ERROR)
	{
		::FailBeError(fWriterCondition);
	}
}


void
CheapReadWriteLock::PlatformDestructor()
{
	::FailBeError(::delete_sem(fReaderCondition));
	::FailBeError(::delete_sem(fWriterCondition));
}


void
CheapReadWriteLock::PlatformBlockWriter()
{
	::FailBeError(::acquire_sem(fWriterCondition));
}


void
CheapReadWriteLock::PlatformUnblockWriter()
{
	::FailBeError(::release_sem(fWriterCondition));
}


void
CheapReadWriteLock::PlatformBlockReader()
{
	::FailBeError(::acquire_sem(fReaderCondition));
}


void
CheapReadWriteLock::PlatformUnblockReaders(long howMany)
{
	::FailBeError(::release_sem_etc(fReaderCondition, howMany, 0));
}

