// ReadLockEntry.cp
// by Eric Berdahl
// Copyright 1996,1997 Eric Berdahl
// All rights reserved
//
// For licensing instructions, see Betelgeuse.h
// For usage instructions, see Synchronization.h


#include "Exceptions.h"
#include "Synchronization.h"


#pragma export on


//------------------------------------------------------------------------------
//	ReadLockEntry implementation
//------------------------------------------------------------------------------


ReadLockEntry::ReadLockEntry(ReadWriteLock* theLock) :
	fLock(theLock),
	fCheapLock(NULL)
{
	::Assertion(theLock != NULL, "ReadLockEntry: constructred with NULL lock");
	theLock->AcquireShared();
}


ReadLockEntry::ReadLockEntry(CheapReadWriteLock* theLock) :
	fLock(NULL),
	fCheapLock(theLock)
{
	::Assertion(theLock != NULL, "ReadLockEntry: constructred with NULL lock");
	theLock->AcquireShared();
}


ReadLockEntry::~ReadLockEntry()
{
	if (fLock != NULL)
	{
		fLock->ReleaseShared();
	}
	else if (fCheapLock != NULL)
	{
		fCheapLock->ReleaseShared();
	}
	else
	{
		::Assertion(false, "ReadLockEntry::~ReadLockEntry: I don't have a lock object");
	}
}


#pragma export off
