// ReserveEntryPPC.cp
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved
//
// For licensing instructions, see Betelgeuse.h
// For usage instructions, see SynchronizationInternal.h


#include "SynchronizationInternal.h"


//------------------------------------------------------------------------------
//	ReserveEntry implementation
//------------------------------------------------------------------------------


asm
ReserveEntryPPC::ReserveEntryPPC(register long* addressToReserve) :
	fReservedLocation(addressToReserve)	// unfortunately, we need both
										// this and the assembly below
										// to actually initialize the const
										// data member
{
	//	Register usage
	//	r3 - this
	//	r4 - addressToReserve

	stw		r4,ReserveEntryPPC.fReservedLocation(r3)	// save the location
	lwarx	r4,r0,r4							// load the contents of addressToReserve
												//    (r0 means 0)
	stw		r4,ReserveEntryPPC.fCachedValue(r3)	// store the value into our cache
	blr
}


ReserveEntryPPC::~ReserveEntryPPC()
{
	// This space intentionally left blank
}


long
ReserveEntryPPC::Get() const
{
	return fCachedValue;
}


asm
bool
ReserveEntryPPC::Set(register long newValue)
{
	//	Register usage on entry
	//	r3 - this
	//	r4 - newValue
	//
	//	Register usage
	//	r3 - result
	//	r4 - newValue
	//	r5 - reserved address
	//	r6 - this

	mr		r6,r3		// save this in  r6 (because r3 must contain the result)
	lwz		r5,ReserveEntryPPC.fReservedLocation(r3)	// fetch the reserved address
	li		r3,1		// pre-load 1 (true)

	stwcx.	r4,r0,r5	// attempt to store the new value into the
						//    reserved address (r0 means 0)
	beq		@success	// the store was successfull, don't bother re-fetching or
						//    resetting the result

	li		r3,0		// load 0 (false)
	lwarx	r4,r0,r5	// remake the reservation

@success
	stw		r4,ReserveEntryPPC.fCachedValue(r6)
	blr
}
