// SynchronizationInternal.h
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved
//
// For licensing and usage instructions, see Betelgeuse.h

#pragma once

// Includes

// Classes defined here

class ReserveEntryPPC;

// Classes used here


//------------------------------------------------------------------------------
//	Global functions
//------------------------------------------------------------------------------

// CompareAndSwap atomically changes variable to hold newValue iff variable
// currently has the value currentValue, returning true if it does. The
// function returns false if variable does not hold newValue. This routine can
// be used to do simple optimistic concurrency.
extern bool CompareAndSwap(long* variable, long currentValue, long newValue);


//------------------------------------------------------------------------------
//	ReserveEntryPPC interface (PPC specific class)
//
//	Constructing a ReserveEntryPPC object caches the long stored at the
//	address and reserves that address. The cached value may be read using the
//	Get member function. Set attempts to write to the reserved address, and
//	then re-reserve the address. Set returns a boolean value, indicating that
//	the store was successful (and therefore that the reserve held). The
//	primary advantage this has over the CompareAndSwap routine is that the
//	fetch of the value is guaranteed to be atomic. With ReserveEntryPPC code
//	can be written to do optimistic concurrency (spin locks) in the following
//	fashion:
//
//	long gValue;
//	...
//	long newValue;
//	ReserveEntryPPC theReserve(&gValue);
//	do
//	{
//		const long initialValue = theReserve.Get();
//		newValue = ModifyValue(initialValue);
//	} while (!theReserve.Set(newValue));
//------------------------------------------------------------------------------


class ReserveEntryPPC
{
public:
	ReserveEntryPPC(long* addressToReserve);
	~ReserveEntryPPC();

	long	Get() const;
	bool	Set(long newValue);

private:
						ReserveEntryPPC(const ReserveEntryPPC& copy);
	ReserveEntryPPC&	operator=(const ReserveEntryPPC& assign);

private:
	long		fCachedValue;
	long* const	fReservedLocation;
};



