// WriteLockEntry.cp
// by Eric Berdahl
// Copyright 1996,1997 Eric Berdahl
// All rights reserved
//
// For licensing instructions, see Betelgeuse.h
// For usage instructions, see Synchronization.h


#include "Exceptions.h"
#include "Synchronization.h"


#pragma export on


//------------------------------------------------------------------------------
//	WriteLockEntry implementation
//------------------------------------------------------------------------------


WriteLockEntry::WriteLockEntry(ReadWriteLock* theLock) :
	fLock(theLock),
	fCheapLock(NULL)
{
	::Assertion(theLock != NULL, "WriteLockEntry: constructred with NULL lock");
	theLock->AcquireExclusive();
}


WriteLockEntry::WriteLockEntry(CheapReadWriteLock* theLock) :
	fLock(NULL),
	fCheapLock(theLock)
{
	::Assertion(theLock != NULL, "WriteLockEntry: constructred with NULL lock");
	theLock->AcquireExclusive();
}


WriteLockEntry::~WriteLockEntry()
{
	if (fLock != NULL)
	{
		fLock->ReleaseExclusive();
	}
	else if (fCheapLock != NULL)
	{
		fCheapLock->ReleaseExclusive();
	}
	else
	{
		::Assertion(false, "WriteLockEntry: Cannot release because I don't have a lock object");
	}
}


#pragma export off
