// CheapReadWriteLockTestProgram.cp
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved


#include "CheapReadWriteLockTestProgram.h"

#include <OS.h>

#include <stdio.h>


//------------------------------------------------------------------------------
// CheapReadWriteLockTestProgram::Data implementation
//------------------------------------------------------------------------------


CheapReadWriteLockTestProgram::Data::Data() :
	fSemaphore(),
	fLock(),
	fCondition(&fLock),
	fNumberReaders(0),
	fNumberWriters(0)
{
	// This space intentionally left blank
}


CheapReadWriteLockTestProgram::Data::Data(long maxNumberReaders) :
	fSemaphore(maxNumberReaders),
	fLock(),
	fCondition(&fLock),
	fNumberReaders(0),
	fNumberWriters(0)
{
	// This space intentionally left blank
}


CheapReadWriteLockTestProgram::Data::~Data()
{
	// This space intentionally left blank
}


void
CheapReadWriteLockTestProgram::Data::PrintResults(long expectedReaders,
	long expectedWriters, const char* label) const
{
	if (label != NULL)
	{
		::printf("%s ", label);
	}

	if (fNumberReaders == expectedReaders  &&
	    fNumberWriters == expectedWriters)
	{
		::printf("PASS\n");
	}
	else
	{
		::printf("FAIL - expected %ld readers and %ld writers; observed %ld readers and %ld writers\n",
		         expectedReaders, expectedWriters,
		         fNumberReaders, fNumberWriters);
	}
	::fflush(stdout);
}


//------------------------------------------------------------------------------
// CheapReadWriteLockTestProgram implementation
//------------------------------------------------------------------------------


CheapReadWriteLockTestProgram::CheapReadWriteLockTestProgram() :
	ThreadProgram()
{
	// This space intentionally left blank
}


CheapReadWriteLockTestProgram::~CheapReadWriteLockTestProgram()
{
	// This space intentionally left blank
}


void
CheapReadWriteLockTestProgram::Run()
{
	const long kDelaySeconds = 1;
	const long kSeconds = 1000000;
	::printf("CheapReadWriteLock test: running (approx %ld seconds)...\n", 19*kDelaySeconds);

	{
		Data theData;
	
		const int kFirstReaders = 3;
		const int kSecondWriters = 1;
		const int kThirdReaders = 1;
		const int kFourthWriters = 2;
		const int kFifthReaders = 1;
	
		for (int i = kFirstReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		theData.PrintResults(kFirstReaders, 0, "CheapReadWriteLock test: simultaneous readers");
	
		for (int i = kSecondWriters; i > 0; --i)
		{
			ThreadHandle theThread(new WriterTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		theData.PrintResults(kFirstReaders, 0, "CheapReadWriteLock test: readers block writers");
	
		for (int i = kThirdReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		theData.PrintResults(kFirstReaders, 0, "CheapReadWriteLock test: waiting writers block readers");
	
		for (int i = kFourthWriters; i > 0; --i)
		{
			ThreadHandle theThread(new WriterTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		for (int i = kFifthReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		theData.PrintResults(kFirstReaders, 0, "CheapReadWriteLock test: complex blocking");
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("CheapReadWriteLock test: single reader releasing ");
		theData.PrintResults(kFirstReaders - 1, 0, NULL);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Broadcast();
		}	
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("CheapReadWriteLock test: readers unblock writers ");
		theData.PrintResults(0, 1, NULL);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("CheapReadWriteLock test: writers unblock all readers ");
		theData.PrintResults(kThirdReaders + kFifthReaders, 0, NULL);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Broadcast();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("CheapReadWriteLock test: writers provide mutex ");
		theData.PrintResults(0, 1, NULL);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("CheapReadWriteLock test: writers unblock writers w/no waiting readers ");
		theData.PrintResults(0, 1, NULL);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		theData.PrintResults(0, 0, "CheapReadWriteLock test: writers release");
	}

	{
		const long kMaxReaders = 2;
		Data theData(kMaxReaders);

		const int kFirstReaders = 3;
		const int kSecondWriters = 1;
		const int kThirdReaders = 2;
	
		for (int i = kFirstReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		theData.PrintResults(kMaxReaders, 0, "CheapReadWriteLock test: maximum readers");
	
		for (int i = kSecondWriters; i > 0; --i)
		{
			ThreadHandle theThread(new WriterTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		theData.PrintResults(kMaxReaders, 0, "CheapReadWriteLock test: readers still block writers");
	
		for (int i = kThirdReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		theData.PrintResults(kMaxReaders, 0, "CheapReadWriteLock test: waiting writers still block readers");
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		theData.PrintResults(kMaxReaders - 1, 0, "CheapReadWriteLock test: single reader releasing");
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Broadcast();
		}	
	
		::snooze(kDelaySeconds*kSeconds);
		theData.PrintResults(0, 1, "CheapReadWriteLock test: readers unblock writers");
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		theData.PrintResults(kMaxReaders, 0, "CheapReadWriteLock test: writers unblock only MAX readers");
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		theData.PrintResults(kMaxReaders, 0, "CheapReadWriteLock test: readers allowed to share");
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Broadcast();
		}

		::snooze(kDelaySeconds*kSeconds);
		theData.PrintResults(0, 0, "CheapReadWriteLock test: readers release");
	}

	::printf("CheapReadWriteLock test: completed.\n");
}


//------------------------------------------------------------------------------
// CheapReadWriteLockTestProgram::ReaderTestProgram implementation
//------------------------------------------------------------------------------


CheapReadWriteLockTestProgram::ReaderTestProgram::ReaderTestProgram(Data* theData) :
	ThreadProgram(),
	fData(theData)
{
	// This space intentionally left blank
}


CheapReadWriteLockTestProgram::ReaderTestProgram::~ReaderTestProgram()
{
	// This space intentionally left blank
}


void
CheapReadWriteLockTestProgram::ReaderTestProgram::Run()
{
	ReadLockEntry trojan1(&fData->fSemaphore);

	MonitorEntry trojan2(&fData->fLock);
	++fData->fNumberReaders;
	fData->fCondition.Wait();
	--fData->fNumberReaders;
}


//------------------------------------------------------------------------------
// CheapReadWriteLockTestProgram::WriterTestProgram implementation
//------------------------------------------------------------------------------


CheapReadWriteLockTestProgram::WriterTestProgram::WriterTestProgram(Data* theData) :
	ThreadProgram(),
	fData(theData)
{
	// This space intentionally left blank
}


CheapReadWriteLockTestProgram::WriterTestProgram::~WriterTestProgram()
{
	// This space intentionally left blank
}


void
CheapReadWriteLockTestProgram::WriterTestProgram::Run()
{
	WriteLockEntry trojan1(&fData->fSemaphore);

	MonitorEntry trojan2(&fData->fLock);
	++fData->fNumberWriters;
	fData->fCondition.Wait();
	--fData->fNumberWriters;
}


