// MonitorConditionTestProgram.cp
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved


#include "MonitorConditionTestProgram.h"

#include <stdio.h>


//------------------------------------------------------------------------------
// MonitorConditionTestProgram::Data implementation
//------------------------------------------------------------------------------


MonitorConditionTestProgram::Data::Data() :
	fProtectedResource(0),
	fLock(),
	fCondition(&fLock)
{
	// This space intentionally left blank
}


MonitorConditionTestProgram::Data::~Data()
{
	// This space intentionally left blank
}


//------------------------------------------------------------------------------
// MonitorConditionTestProgram implementation
//------------------------------------------------------------------------------


MonitorConditionTestProgram::MonitorConditionTestProgram() :
	ThreadProgram()
{
	// This space intentionally left blank
}


MonitorConditionTestProgram::~MonitorConditionTestProgram()
{
	// This space intentionally left blank
}


void
MonitorConditionTestProgram::Run()
{
	const long kDelaySeconds = 2;
	::printf("MonitorCondition test: running (approx %ld seconds)...\n", 3*kDelaySeconds);

	const long kNumberTests = 10;

	Data theData;

		// Launch a number of threads which will wait on the condition
		// and wait for a while to allow them all to enter the monitor
		// and block.
	for (long i = kNumberTests; i > 0; --i)
	{
		ThreadHandle theThread(new TestProgram(&theData));
	}

	const long kSeconds = 1000000;
	::snooze(kDelaySeconds*kSeconds);

		// Check to see how many are now blocked (using the
		// longint as a proxy.
	::printf("MonitorCondition test: Wait ");
	if (theData.fProtectedResource == kNumberTests)
	{
		::printf("PASS\n");
	}
	else
	{
		::printf("FAIL - only %ld of %ld threads are blocked\n",
		         theData.fProtectedResource, kNumberTests);
	}

		// Now, Notify the condition and verify that exactly
		// one person is woken up.
	{
		MonitorEntry trojan(&theData.fLock);
		theData.fCondition.Notify();
	}
	::snooze(kDelaySeconds*kSeconds);
	::printf("MonitorCondition test: Notify ");
	if (theData.fProtectedResource == kNumberTests - 1)
	{
		::printf("PASS\n");
	}
	else
	{
		::printf("FAIL - %ld threads awoke (expected 1)\n",
		         kNumberTests - theData.fProtectedResource);
	}

		// Now, Broadcast the condition and verify that everyone
		// has been woken up.
	{
		MonitorEntry trojan(&theData.fLock);
		theData.fCondition.Broadcast();
	}
	::snooze(kDelaySeconds*kSeconds);
	::printf("MonitorCondition test: Broadcast ");
	if (theData.fProtectedResource == 0)
	{
		::printf("PASS\n");
	}
	else
	{
		::printf("FAIL - %ld threads are still blocked\n",
		         theData.fProtectedResource);
	}

	::printf("MonitorCondition test: completed.\n");
}


//------------------------------------------------------------------------------
// MonitorConditionTestProgram::TestProgram implementation
//------------------------------------------------------------------------------


MonitorConditionTestProgram::TestProgram::TestProgram(Data* theData) :
	ThreadProgram(),
	fData(theData)
{
	// This space intentionally left blank
}


MonitorConditionTestProgram::TestProgram::~TestProgram()
{
	// This space intentionally left blank
}


void
MonitorConditionTestProgram::TestProgram::Run()
{
	MonitorEntry trojan(&fData->fLock);
	++fData->fProtectedResource;
	fData->fCondition.Wait();
	--fData->fProtectedResource;
}

