// MonitorLockTestProgram.h
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved

#pragma once

// Includes

#include <Synchronization.h>
#include <Task.h>

// Classes defined here

class MonitorLockTestProgram;


// Classes used here


//------------------------------------------------------------------------------
// MonitorLockTestProgram interface
//------------------------------------------------------------------------------


class MonitorLockTestProgram : public ThreadProgram
{
public:
	MonitorLockTestProgram();
	virtual	~MonitorLockTestProgram();

	virtual void	Run();

private:
	void	TestEntry(long numberSeconds);
	void	TestReentry(long numberSeconds);

private:
	struct Data
	{
				Data();
				~Data();
		void	PrintResults() const;

		const long			fIncrement;
		const long			fNumberIterations;
		long				fProtectedResource;
		long				fNumberExecutions;
		MonitorLock			fLock;
		MonitorCondition	fCondition;
		bool				fSuccess;
		bool				fQuitting;
	};

private:
	class TestProgram : public ThreadProgram
	{
	public:
						TestProgram(Data* theData, bool increment);
		virtual			~TestProgram();

		// Remember that Run is pure virtual in the base class

	protected:
		void			TestIteration();
		void			Pause();

		Data* const	fData;
	private:
		int			fSignum;
	};

	class EntryTestProgram : public TestProgram
	{
	public:
						EntryTestProgram(Data* theData, bool increment);
		virtual			~EntryTestProgram();

		virtual void	Run();
	};

	class ReentryTestProgram : public TestProgram
	{
	public:
						ReentryTestProgram(Data* theData, bool increment);
		virtual			~ReentryTestProgram();

		virtual void	Run();
	};

	class ReentryTimerProgram : public TestProgram
	{
	public:
						ReentryTimerProgram(Data* theData);
		virtual			~ReentryTimerProgram();

		virtual void	Run();
	};
};

