// ReadWriteLockTestProgram.cp
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved


#include "ReadWriteLockTestProgram.h"

#include <OS.h>

#include <stdio.h>


//------------------------------------------------------------------------------
// ReadWriteLockTestProgram::Data implementation
//------------------------------------------------------------------------------


ReadWriteLockTestProgram::Data::Data() :
	fSemaphore(),
	fLock(),
	fCondition(&fLock),
	fNumberReaders(0),
	fNumberWriters(0)
{
	// This space intentionally left blank
}


ReadWriteLockTestProgram::Data::Data(long maxNumberReaders) :
	fSemaphore(maxNumberReaders),
	fLock(),
	fCondition(&fLock),
	fNumberReaders(0),
	fNumberWriters(0)
{
	// This space intentionally left blank
}


ReadWriteLockTestProgram::Data::~Data()
{
	// This space intentionally left blank
}


void
ReadWriteLockTestProgram::Data::PrintResults(long expectedReaders,
	long expectedWriters) const
{
	if (fNumberReaders == expectedReaders  &&
	    fNumberWriters == expectedWriters)
	{
		::printf("PASS\n");
	}
	else
	{
		::printf("FAIL - expected %ld readers and %ld writers; observed %ld readers and %ld writers\n",
		         expectedReaders, expectedWriters,
		         fNumberReaders, fNumberWriters);
	}
	::fflush(stdout);
}


//------------------------------------------------------------------------------
// ReadWriteLockTestProgram implementation
//------------------------------------------------------------------------------


ReadWriteLockTestProgram::ReadWriteLockTestProgram() :
	ThreadProgram()
{
	// This space intentionally left blank
}


ReadWriteLockTestProgram::~ReadWriteLockTestProgram()
{
	// This space intentionally left blank
}


void
ReadWriteLockTestProgram::Run()
{
	const long kDelaySeconds = 1;
	const long kSeconds = 1000000;
	::printf("ReadWriteLock test: running (approx %ld seconds)...\n", 19*kDelaySeconds);

	{
		Data theData;
	
		const int kFirstReaders = 3;
		const int kSecondWriters = 1;
		const int kThirdReaders = 1;
		const int kFourthWriters = 2;
		const int kFifthReaders = 1;
	
		for (int i = kFirstReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		::printf("ReadWriteLock test: simultaneous readers ");
		theData.PrintResults(kFirstReaders, 0);
	
		for (int i = kSecondWriters; i > 0; --i)
		{
			ThreadHandle theThread(new WriterTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		::printf("ReadWriteLock test: readers block writers ");
		theData.PrintResults(kFirstReaders, 0);
	
		for (int i = kThirdReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		::printf("ReadWriteLock test: waiting writers block readers ");
		theData.PrintResults(kFirstReaders, 0);
	
		for (int i = kFourthWriters; i > 0; --i)
		{
			ThreadHandle theThread(new WriterTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		for (int i = kFifthReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		::printf("ReadWriteLock test: complex blocking ");
		theData.PrintResults(kFirstReaders, 0);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: single reader releasing ");
		theData.PrintResults(kFirstReaders - 1, 0);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Broadcast();
		}	
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: readers unblock writers ");
		theData.PrintResults(0, 1);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: writers unblock all readers ");
		theData.PrintResults(kThirdReaders + kFifthReaders, 0);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Broadcast();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: writers provide mutex ");
		theData.PrintResults(0, 1);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: writers unblock writers w/no waiting readers ");
		theData.PrintResults(0, 1);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: writers release ");
		theData.PrintResults(0, 0);
	}

	{
		const long kMaxReaders = 2;
		Data theData(kMaxReaders);

		const int kFirstReaders = 3;
		const int kSecondWriters = 1;
		const int kThirdReaders = 2;
	
		for (int i = kFirstReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		::printf("ReadWriteLock test: maximum readers ");
		theData.PrintResults(kMaxReaders, 0);
	
		for (int i = kSecondWriters; i > 0; --i)
		{
			ThreadHandle theThread(new WriterTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		::printf("ReadWriteLock test: readers still block writers ");
		theData.PrintResults(kMaxReaders, 0);
	
		for (int i = kThirdReaders; i > 0; --i)
		{
			ThreadHandle theThread(new ReaderTestProgram(&theData));
		}
		::snooze(kDelaySeconds*kSeconds);
	
		::printf("ReadWriteLock test: waiting writers still block readers ");
		theData.PrintResults(kMaxReaders, 0);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: single reader releasing ");
		theData.PrintResults(kMaxReaders - 1, 0);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Broadcast();
		}	
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: readers unblock writers ");
		theData.PrintResults(0, 1);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: writers unblock only MAX readers ");
		theData.PrintResults(kMaxReaders, 0);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Notify();
		}
	
		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: readers allowed to share ");
		theData.PrintResults(kMaxReaders, 0);
	
		{
			MonitorEntry trojan(&theData.fLock);
			theData.fCondition.Broadcast();
		}

		::snooze(kDelaySeconds*kSeconds);
		::printf("ReadWriteLock test: readers release ");
		theData.PrintResults(0, 0);
	}

	::printf("ReadWriteLock test: completed.\n");
}


//------------------------------------------------------------------------------
// ReadWriteLockTestProgram::ReaderTestProgram implementation
//------------------------------------------------------------------------------


ReadWriteLockTestProgram::ReaderTestProgram::ReaderTestProgram(Data* theData) :
	ThreadProgram(),
	fData(theData)
{
	// This space intentionally left blank
}


ReadWriteLockTestProgram::ReaderTestProgram::~ReaderTestProgram()
{
	// This space intentionally left blank
}


void
ReadWriteLockTestProgram::ReaderTestProgram::Run()
{
	ReadLockEntry trojan1(&fData->fSemaphore);

	MonitorEntry trojan2(&fData->fLock);
	++fData->fNumberReaders;
	fData->fCondition.Wait();
	--fData->fNumberReaders;
}


//------------------------------------------------------------------------------
// ReadWriteLockTestProgram::WriterTestProgram implementation
//------------------------------------------------------------------------------


ReadWriteLockTestProgram::WriterTestProgram::WriterTestProgram(Data* theData) :
	ThreadProgram(),
	fData(theData)
{
	// This space intentionally left blank
}


ReadWriteLockTestProgram::WriterTestProgram::~WriterTestProgram()
{
	// This space intentionally left blank
}


void
ReadWriteLockTestProgram::WriterTestProgram::Run()
{
	WriteLockEntry trojan1(&fData->fSemaphore);

	MonitorEntry trojan2(&fData->fLock);
	++fData->fNumberWriters;
	fData->fCondition.Wait();
	--fData->fNumberWriters;
}


