// Exceptions.Be.cp
// by Eric Berdahl
// Copyright 1997 Eric Berdahl
// All rights reserved
//
// For licensing instructions, see Betelgeuse.h
// For usage instructions, see Exceptions.h


// This is a good idea for debugging purposes. For production code
// you may choose to not have even these routines do debug checks.
// Your mileage may vary.
#define DEBUG 1

#include <Synchronization.h>

#include "Exceptions.h"

#include <Debug.h>
#include <Errors.h>

#include <stdio.h>
#include <va_list.h>


//------------------------------------------------------------------------------
// Global functions
//------------------------------------------------------------------------------


void
FailBeError(long errorCode)
{
		// Quick return -- no error
	if (errorCode == B_NO_ERROR)
		return;

	// Ideally, we'd like to transmute the error code into an exception
	// class, thrown here. However, I really don't have a good strategy for
	// exceptions yet, so we'll deal with something simple.
	::fprintf(stderr, "FailBeError found error code %ld\n", errorCode);
	::fflush(stderr);
	throw BeException(errorCode);
}


#if BetelgeuseDebug

void
Assertion(bool theCondition, const char* theMessage)
{
	if (!theCondition)
	{
		const char* hackToPreventCompilerComplaints = theMessage;
		DEBUGGER(theMessage);
	}
}


static MonitorLock	gDebugPrintfLock;

void
DebugPrintf(const char* format, ...)
{
	MonitorEntry trojan(&gDebugPrintfLock);
	::vfprintf(stderr, format, __va_start(format));
	::fflush(stderr);
}

#endif
